use rusqlite::{
    types::{FromSql, FromSqlError, FromSqlResult, ToSqlOutput, Value, ValueRef},
    ToSql,
};
use serde::{Deserialize, Serialize};

pub struct SqlJson<T>(pub T);

impl<T> SqlJson<T> {
    pub fn into_inner(self) -> T {
        self.0
    }
}

impl<T> FromSql for SqlJson<T>
where
    for<'de> T: Deserialize<'de>,
{
    fn column_result(value: ValueRef<'_>) -> FromSqlResult<Self> {
        if let ValueRef::Text(text) = value {
            let deser_value: T =
                serde_json::from_slice(text).map_err(|err| FromSqlError::Other(Box::new(err)))?;
            Ok(SqlJson(deser_value))
        } else {
            Err(FromSqlError::InvalidType)
        }
    }
}

impl<T> ToSql for SqlJson<T>
where
    T: Serialize,
{
    fn to_sql(&self) -> rusqlite::Result<ToSqlOutput<'_>> {
        serde_json::to_string(&self.0)
            .map(|serialized| ToSqlOutput::Owned(Value::Text(serialized)))
            .map_err(|err| rusqlite::Error::ToSqlConversionFailure(Box::new(err)))
    }
}
